use crate::{twitter::get_tweet_media, ScootalooError};
use egg_mode::tweet::Tweet;
use elefren::prelude::*;
use log::{error, warn};
use reqwest::Url;
use std::{borrow::Cow, error::Error};
use tokio::{
    fs::{create_dir_all, remove_file, File},
    io::copy,
};

/// Generate associative table between media ids and tweet extended entities
pub async fn generate_media_ids(
    tweet: &Tweet,
    cache_path: &str,
    mastodon: &Mastodon,
) -> (String, Vec<String>) {
    let mut media_ids: Vec<String> = vec![];
    let mut media_url: String = "".to_string();

    if let Some(m) = &tweet.extended_entities {
        for media in &m.media {
            // attribute the media url
            media_url = media.url.clone();
            let local_tweet_media_path = match get_tweet_media(media, cache_path).await {
                Ok(m) => m,
                Err(e) => {
                    error!("Cannot get tweet media for {}: {}", &media.url, e);
                    continue;
                }
            };

            let mastodon_media_ids = match mastodon
                .media(Cow::from(local_tweet_media_path.to_owned()))
            {
                Ok(m) => {
                    remove_file(&local_tweet_media_path).await.unwrap_or_else(|e|
                        warn!("Attachment for {} has been uploaded, but I’m unable to remove the existing file: {}", &local_tweet_media_path, e));
                    m.id
                }
                Err(e) => {
                    error!(
                        "Attachment {} cannot be uploaded to Mastodon Instance: {}",
                        &local_tweet_media_path, e
                    );
                    // file is no longer useful, deleting
                    remove_file(&local_tweet_media_path).await.unwrap_or_else(|e|
                        warn!("Attachment for {} has been uploaded, but I’m unable to remove the existing file: {}", &local_tweet_media_path, e));
                    continue;
                }
            };

            media_ids.push(mastodon_media_ids);
        }
    }

    (media_url, media_ids)
}

/// Gets and caches Twitter Media inside the determined temp dir
pub async fn cache_media(u: &str, t: &str) -> Result<String, Box<dyn Error>> {
    // create dir
    create_dir_all(t).await?;

    // get file
    let mut response = reqwest::get(u).await?;

    // create local file
    let url = Url::parse(u)?;
    let dest_filename = url
        .path_segments()
        .ok_or_else(|| {
            ScootalooError::new(&format!(
                "Cannot determine the destination filename for {}",
                u
            ))
        })?
        .last()
        .ok_or_else(|| {
            ScootalooError::new(&format!(
                "Cannot determine the destination filename for {}",
                u
            ))
        })?;

    let dest_filepath = format!("{}/{}", t, dest_filename);

    let mut dest_file = File::create(&dest_filepath).await?;

    while let Some(chunk) = response.chunk().await? {
        copy(&mut &*chunk, &mut dest_file).await?;
    }

    Ok(dest_filepath)
}

#[cfg(test)]
mod tests {
    use super::*;

    use std::{fs::remove_dir_all, path::Path};

    const TMP_DIR: &'static str = "/tmp/scootaloo_test";

    #[tokio::test]
    async fn test_cache_media() {
        let dest = cache_media(
            "https://forum.nintendojo.fr/styles/prosilver/theme/images/ndfr_casual.png",
            TMP_DIR,
        )
        .await
        .unwrap();

        assert!(Path::new(&dest).exists());

        remove_dir_all(TMP_DIR).unwrap();
    }
}
